" Vim syntax file
" Language:             Codex
" Maintainter:          Thaddee Tyl <thaddee.tyl@gmail.com>
" ----------------------------------------------------------------------------

if version < 600
  syntax clear
elseif exists("b:current_syntax")
  finish
endif


syn keyword rtConst yes no minPosint maxPosint minInt maxInt leastPosApprox leastNegApprox minApprox maxApprox posInfinity negInfinity posZero negZero notANumber
syn keyword rtConditional if then else cond
syn keyword rtRepeat  while loop for
syn keyword rtType byte char int8 posint8 int16 posint16 int32 posint32 int64 posint64 float32 float64 bool int posint bigint approx
syn keyword rtTodo  TODO todo FIXME XXX contained
syn match   rtComment "#.*$" contains=rtTodo
syn match   rtPreProc /^#!.*$/
syn match   rtPreProc /@/
syn match   rtType    /=\|:/
syn match   rtOperator /\[\!\~\%\^\&\*\=\-\+\<\>\]/
syn match   rtSpecial /\./

hi def link rtComment Comment
hi def link rtConst Boolean
hi def link rtFunction Function
hi def link rtConditional Conditional
hi def link rtRepeat Repeat
hi def link rtOperator Operator
hi def link rtKeyword Keyword
hi def link rtPreProc PreProc
hi def link rtType Type
hi def link rtDelimiter Delimiter
hi def link rtTodo Todo
hi def link rtSpecial Special
hi def link rtStringSym Special
hi def link rtEscape Special


" Strings
syn match       rtEscapeOctal       display contained "\\o\x\{3}"
syn match       rtEscapeC           display contained +\\[abfnrtv\\'"]+
syn match       rtEscapeX           display contained "\\x\x\{2}"
syn match       rtEscapeU           display contained "\\u\x\{4}"
syn match       rtEscapeBigU        display contained "\\U\x\{8}"
syn match       rtEscapeError       display contained +\\[^0-7xuUabfnrtv\\'"]+

hi def link     rtEscapeOctal       rtSpecialString
hi def link     rtEscapeC           rtSpecialString
hi def link     rtEscapeX           rtSpecialString
hi def link     rtEscapeU           rtSpecialString
hi def link     rtEscapeBigU        rtSpecialString
hi def link     rtSpecialString     Special
hi def link     rtEscapeError       Error

" Strings and their contents
syn cluster     rtStringGroup       contains=rtEscapeOctal,rtEscapeC,rtEscapeX,rtEscapeU,rtEscapeBigU,rtEscapeError
syn region      rtString            start=+"+ skip=+\\\\\|\\"+ end=+"+ contains=@rtStringGroup
syn region      rtStringSingle      start=+'+ skip=+\\\\\|\\'+ end=+'+ contains=@rtStringGroup
syn region      rtRawString         start=+`+ end=+`+

hi def link     rtString            String
hi def link     rtStringSingle      String
hi def link     rtRawString         String

let rt_highlight_all = 1
if exists("rt_highlight_all")
  let rt_highlight_numbers = 1
  let rt_highlight_space_errors = 1
endif

if exists("rt_highlight_numbers")
  " Integers
  syn match     rtDecimalInt     "\<\(\d\|_\)\+\>"
  syn match     rtHexadecimalInt "\<0x[\x_]\+\>"
  syn match     rtOctalInt       "\<0o[\o_]\+\>"
  syn match     rtOctalError     "\<0o[\o_]*[89]\d*\>"

  hi def link   rtDecimalInt     Integer
  hi def link   rtHexadecimalInt Integer
  hi def link   rtOctalInt       Integer
  hi def link   Integer          Number

  " Floating point
  syn match     rtFloat          "\<\(\d\|_\)\+\.\d*\([Ee][-+]\(\d\|_\)\+\)\?\>"
  syn match     rtFloat          "\<\.\(\d\|_\)\+\([Ee][-+]\(\d\|_\)\+\)\?\>"
  syn match     rtFloat          "\<\(\d\|_\)\+[Ee][-+]\(\d\|_\)\+"

  hi def link   rtFloat          Float
endif

if exists("rt_highlight_space_errors")
  " trailing whitespace 
  syn match   rtError   display excludenl "\S\s\+$"ms=s+1
  " mixed tabs and spaces
  syn match   rtError   display " \+\t"
  syn match   rtError   display "\t\+ "
endif

hi def link rtError Error


" automatic sync
syn sync maxlines=200

let b:current_syntax = "root"

" vim: sw=2 sts=2 ts=8 noet:
